<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Full Page Cache Warmer for Magento 2
 */

namespace Amasty\Fpc\Model\Queue\Combination\Context;

use Amasty\Fpc\Model\Config;
use Amasty\Groupcat\Model\ResourceModel\Rule\CollectionFactory;
use Amasty\Groupcat\Model\Rule;
use GuzzleHttp\RequestOptions;
use Magento\Framework\App\Http\Context;
use Magento\Framework\Module\Manager;
use Magento\Framework\ObjectManagerInterface;

class GroupCatRuleCombination implements CombinationSourceInterface
{
    /**
     * @var Config
     */
    private $configProvider;

    /**
     * @var Manager
     */
    private $moduleManager;

    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;

    public function __construct(
        Config $configProvider,
        ObjectManagerInterface $objectManager,
        Manager $moduleManager
    ) {
        $this->configProvider = $configProvider;
        $this->moduleManager = $moduleManager;
        $this->objectManager = $objectManager;
    }

    public function getVariations(): array
    {
        $variations = [];

        if ($this->moduleManager->isEnabled('Amasty_Groupcat')) {
            $rules = [];
            $collectionFactory = $this->objectManager->get(CollectionFactory::class);
            $collection = $collectionFactory->create();
            $collection->addIsActiveFilter();
            $collection->getSelect()
                ->reset('columns')
                ->columns('main_table.rule_id');

            foreach ($collection->getItems() as $item) {
                $ruleStoreIds = $item->getStoreIds();
                if (count($ruleStoreIds) === 1 && (int)$ruleStoreIds[0] === 0) {
                    $ruleStoreIds = $this->configProvider->getStores();
                }

                $ruleCustomerGroups = $item->getCustomerGroupIds();

                foreach ($ruleStoreIds as $store) {
                    if (!isset($rules[$store])) {
                        $rules[$store] = [];
                    }

                    foreach ($ruleCustomerGroups as $customerGroup) {
                        if (!isset($rules[$store][$customerGroup])) {
                            $rules[$store][$customerGroup] = $item->getRuleId();
                        } else {
                            $rules[$store][$customerGroup] .= '_' . $item->getRuleId();
                        }
                    }
                }
            }

            foreach ($rules as $store => $customerGroups) {
                foreach ($customerGroups as $customerGroup => $ruleIds) {
                    $variations[] = [
                        'crawler_store' => $store,
                        'crawler_customer_group' => $customerGroup,
                        'crawler_amsty_groupcat_rule' => $ruleIds
                    ];
                }
            }
        }

        return $variations;
    }

    public function getCombinationKey(): string
    {
        return 'crawler_amsty_groupcat_rule';
    }

    public function modifyRequest(array $combination, array &$requestParams, Context $context)
    {
        if ($this->moduleManager->isEnabled('Amasty_Groupcat')) {
            $rules = $combination[$this->getCombinationKey()] ?? '';

            $requestParams[RequestOptions::HEADERS]['X-Amasty-Crawler-Customer-Group-Catalog'] = $rules;
            $context->setValue(
                Rule::CACHE_TAG,
                $rules,
                ''
            );
        }
    }

    public function prepareLog(array $crawlerLogData, array $combination): array
    {
        if ($this->moduleManager->isEnabled('Amasty_Groupcat')) {
            $rules = $combination[$this->getCombinationKey()] ?? '';
            $crawlerLogData['rule'] = $rules;
        }

        return $crawlerLogData;
    }
}
